# -*- coding: utf-8 -*-
import requests
import logging
import json
from os import path
from datadis.validators import validar_contrato, validar_autoconsumo
from datadis.adaptors import adaptar_contrato, adaptar_maximas_potencia, adaptar_estado, adaptar_autoconsumo
from datetime import datetime

BASE_URL = "https://apihsdistribuidoras.asemeservicios.com"
HEADER = {
    'Content-Type': 'application/json', 'Accept-Charset': 'utf-8', 'charset': 'utf-8'
}
logger = logging.getLogger('Datadis Webservice')

class DatadisWebserviceController(object):

    def __init__(self):
        pass

    @property
    def templates(self):
        return path.join(path.dirname(path.realpath(__file__)), 'templates/')

    @property
    def url_autenticar(self):
        return BASE_URL + '/autenticar'

    @property
    def url_contrato(self):
        return BASE_URL + '/contrato'

    @property
    def url_eliminar_contrato(self):
        return BASE_URL + '/contrato/{nif}/{cups}'

    @property
    def url_bloquear_consumidor(self):
        return BASE_URL + '/consumidor/bloquear'

    @property
    def url_maximas_potencia(self):
        return BASE_URL + '/maximaspotencia'

    @property
    def url_estado(self):
        return BASE_URL + '/estado/{timestamp}/{guid}'

    @property
    def url_autoconsumo(self):
        return BASE_URL + '/autoconsumo'

    def autenticar(self, user, password):
        """Autenticarse en el sistema DATADIS.
        Enviar un diccionario con las claves requeridas indicadas a continuacion
        user: Usuario
        password: Clave
        """
        user = str(user).zfill(4)
        r = requests.post(self.url_autenticar, headers=HEADER, json={'usuario': user, 'contraseña': password})
        if r.status_code == 200:
            resp_data = r.json()
            api_token = resp_data.get('token', '')
            username = resp_data.get('usuario', '')
            logger.info('User {} is now connected'.format(username))
            self.token = 'Bearer ' + api_token
            self.user = username
            HEADER['Authorization'] = self.token
            return resp_data
        else:
            raise Exception(r.content)

    def contrato(self, data, method='POST'):
        """Publicar contrato al sistema DATADIS.
        Si el contrato ya existe en el sistema, se modificara
        Enviar un diccionario con las claves requeridas indicadas a continuacion
        data: {
            "nif": "",
            "nombre": "",
            "comercializadora": "",
            "tensionConexion": "",
            "tarifaAcceso": "",
            "discriminacionHoraria": "",
            "tipoPunto": "",
            "modoControlPotencia": "",
            "fechaInicioContrato": "",
            "CNAE": "",
            "cups": "",
            "distribuidora": "",
            "codigoPostal": "",
            "provincia": "",
            "municipio": ""
            "potenciasContratadas": Opcional: lista de potencias ordenadas por periodo o dicionario, o string P1: x P2: y
        }
        method: 'POST' por defecto, utilizar 'DELETE' para eliminar contrato
        return: dict {'guid': identificador de la peticion, 'timestamp': marca de tiempo}
        """
        if method.upper() == 'POST':
            data = adaptar_contrato(data)
            validar_contrato(data)

            with open(self.templates + 'contrato.json') as json_template:
                template = json.load(json_template)
            for key in data.keys():
                if key in template['contrato']:
                    template['contrato'].update({key: data[key]})
                if key in template['puntoSuministro']:
                    template['puntoSuministro'].update({key: data[key]})
                if key in template['titular']:
                    template['titular'].update({key: data[key]})
                if key == 'potenciasContratadas':
                    template['potenciasContratadas'] = data[key]
                if key in ('cau', 'coeficienteReparto'):
                    node_key = 'autoConsumo'
                    if node_key not in template:
                        template[node_key] = {key: data[key]}
                    else:
                        template[node_key].update({key: data[key]})
            r = requests.post(self.url_contrato, headers=HEADER, json=template)
            if r.status_code == 200:
                return r.json()
            elif r.status_code == 202:
                return r.json()
            elif r.status_code == 422:
                raise Exception("No se ha podido publicar el contrato: \n{}".format(r.content))
            else:
                raise Exception("No se ha podido publicar el contrato {}".format(r.status_code))
        if method.upper() == 'DELETE':
            return self.eliminar_contrato(data)

    def eliminar_contrato(self, data):
        """Eliminar un contrato del sistema DATADIS.
        El sistema borrara todos los contratos del mismo NIF y CUPS
        Enviar un diccionario con las claves requeridas indicadas a continuacion
        data: {
            "nif": "",
            "cups": ""
        }
        """
        if 'nif' in data and 'cups' in data:
            r = requests.delete(self.url_eliminar_contrato.format(**data), headers=HEADER)
            if r.status_code == 200:
                return r.json()
            else:
                raise Exception("No se ha podido eliminar el contrato: {}".format(r.status_code))
        else:
            raise KeyError("nif y/o cups no especificados!")

    def maximas_potencia(self, data):
        """Publicar maximetros al sistema DATADIS.
        Enviar un diccionario con las siguiente estructura:
        data: {
            "cups": CON 22 CARACTERES:
            [
                {
                    "fecha": AAAA-MM-DD HH:MM, si no se especifica la hora se utilizara 00:00,
                    "medida": kWh,
                    "periodo": 1, 2, 3, 4, 5 o 6
                }
            ]
        }
        return: dict {'guid': identificador de la peticion, 'timestamp': marca de tiempo}
        """
        with open(self.templates + 'maximaspotencia.json') as json_template:
            template = json.load(json_template)
        cups = data.keys()[0]
        template['cups'] = cups
        for maximeter in data[cups]:
            maximeter = adaptar_maximas_potencia(maximeter)
            template['registros'].append(
                {
                    'fecha': maximeter['fecha'],
                    'hora': maximeter['hora'],
                    'medida': maximeter['medida'],
                    'periodo': maximeter['periodo']
                }
            )
        r = requests.post(self.url_maximas_potencia, headers=HEADER, json=template)
        if r.status_code == 200:
            return r.json()
        elif r.status_code == 202:
            return r.json()
        elif r.status_code == 422:
            raise Exception("No se han podido cargar los maximetros: \n{}".format(r.content))
        else:
            raise Exception("No se han podido cargar los maximetros: {}".format(r.status_code))

    def bloquear_consumidor(self, nif):
        """Bloquear el acceso a la informacion del sistema DATADIS a un consumidor.
        nif: string indicando el NIF del consumidor
        """
        data = {'nif': nif, 'bloqueado': 'true'}
        r = requests.post(self.url_bloquear_consumidor(), headers=HEADER, json=json.dumps(data))
        if r.status_code == 200:
            return r.json()
        else:
            raise Exception("No se ha podido bloquear el acceso al consumidor: {}".format(r.status_code))

    def estado(self, data):
        """Consultar el estado de una peticion enviada al sistema DATADIS.
        Enviar un diccionario con las siguientes claves:
        data: {
            "guid": 'Codigo de la peticion,
            "timestamp": "Fecha de la peticion"
        }
        return: []
        """
        data = adaptar_estado(data)
        r = requests.get(self.url_estado.format(**data), headers=HEADER)
        if r.status_code == 200:
            resp_data = r.json()
            if resp_data["erroresAcumulados"]:
                raise Exception("Errores Acumulados: \n{}".format(resp_data))
            return resp_data
        elif r.status_code == 422:
            raise Exception("Error en el formato de datos de la peticion: \n{}".format(r.content))
        else:
            raise Exception("No se ha podido consultar el estado de la peticion: {}".format(r.status_code))

    def autoconsumo(self, data):
        """Publicar autoconsumo al sistema DATADIS.
        Enviar un diccionario con las claves requeridas indicadas a continuacion
        data: {
            "cau": "CAU",
            "tipoAutoConsumo": "",
            "seccion": "1 o 2",
            "subseccion": "",
            "potenciaInstaladaGeneracion": "",
        }
        return: dict {'guid': identificador de la peticion, 'timestamp': marca de tiempo}
        """
        if True:
            data = adaptar_autoconsumo(data)
            validar_autoconsumo(data)

            with open(self.templates + 'autoconsumo.json') as json_template:
                template = json.load(json_template)
            for key in data.keys():
                if key in template:
                    template.update({key: data[key]})
            r = requests.post(self.url_autoconsumo, headers=HEADER, json=template)
            if r.status_code == 200:
                return r.json()
            elif r.status_code == 202:
                return r.json()
            elif r.status_code == 422:
                raise Exception("No se ha podido publicar el autoconsumo: \n{}".format(r.content))
            else:
                raise Exception("No se ha podido publicar el autoconsumo {}".format(r.status_code))



# -*- coding: utf-8 -*-
from re import findall
from datetime import datetime

REQUIRED_CONTRATO_KEYS = [
    'comercializadora', 'tensionConexion', 'tarifaAcceso', 'discriminacionHoraria', 'tipoPunto', 'modoControlPotencia',
    'fechaInicioContrato', 'nif', 'nombre', 'cups', 'distribuidora', 'codigoPostal', 'provincia', 'municipio', 'CNAE'
]
REQUIRED_MAXIMAS_POTENCIA_KEYS = [
    'medida', 'fecha', 'periodo'
]

REQUIRED_AUTOCONSUMO_KEYS = [
    'cau', 'tipoAutoConsumo', 'seccion', 'subseccion'
]

def adaptar_contrato(data):
    for key in REQUIRED_CONTRATO_KEYS:
        if key not in data:
            raise KeyError("Clave requerida {} no encontrada!".format(key))
    if len(str(data['municipio'])) != 3:
        data['municipio'] = str(data['municipio'])[-3:]
    if not data['CNAE']:
        data.update({'CNAE': ""})
    else:
        data.update({'CNAE': str(data['CNAE']).zfill(4)})
    data.update({
        'comercializadora': str(data['comercializadora'].zfill(4)),
        'distribuidora': str(data['distribuidora'].zfill(4)),
        'tarifaAcceso': str(data['tarifaAcceso'].upper()),
        'tensionConexion': str(data['tensionConexion'].upper()),
        'discriminacionHoraria': str(data['discriminacionHoraria'].upper()),
        'municipio': str(data['municipio']).zfill(3),
        'provincia': str(data['provincia']).zfill(2),
        'codigoPostal': str(data['codigoPostal']).zfill(5),
        'tipoPunto': int(data['tipoPunto']),
        'nif': str(data['nif']).replace('ES', '')
    })
    if not isinstance(data['modoControlPotencia'], int):
        control_potencia = 1 if 'max' in data['modoControlPotencia'] else 2
        data.update({'modoControlPotencia': control_potencia})
    if 'potenciasContratadas' in data:
        if isinstance(data['potenciasContratadas'], (dict)):
            potencias = []
            for val in data['potenciasContratadas'].values():
                potencias.append(float(val))
            data['potenciasContratadas'] = potencias
        elif not isinstance(data['potenciasContratadas'], (list, dict)):
            potencias = data['potenciasContratadas']
            potencias_keys = findall('P\d', potencias)
            for pot_key in potencias_keys:
                potencias = potencias.replace('{}: '.format(pot_key), '')
            potencias = potencias.split(' ')
            for i, pot in enumerate(potencias):
                potencias[i] = float(potencias[i])
            data['potenciasContratadas'] = potencias
    if 'cau' in data:
        data.update({'coeficienteReparto': float(data['coeficienteReparto'])})
    return data

def adaptar_maximas_potencia(data):
    for key in REQUIRED_MAXIMAS_POTENCIA_KEYS:
        if key not in data:
            raise KeyError("Clave requerida {} no encontrada!".format(key))

    hour_mask = "%H:%M"
    date_mask = "%Y-%m-%d"
    try:
        ts = datetime.strptime(data['fecha'], date_mask + ' ' + hour_mask + ":%S")
    except (ValueError, TypeError):
        try:
            ts = datetime.strptime(data['fecha'], date_mask + ' ' + hour_mask)
        except (ValueError, TypeError):
            try:
                ts = datetime.strptime(data['fecha'], date_mask)
            except:
                raise Exception("Formato {} fecha incorrecto -> AAAA/mm/dd o AAAA/mm/dd HH:MM o AAAA/mm/dd HH:MM:SS".format(data['fecha']))
    fecha = ts.strftime(date_mask)
    hora = ts.strftime(hour_mask)
    medida = "%.3f" % round(data['medida'], 3)
    medida = float(medida)
    periodo = int(data['periodo'])
    if periodo not in [1, 2, 3, 4, 5, 6]:
        raise Exception("Periodo {} incorrecto. Valores permitidos 1, 2, 3, 4, 5, 6".format(periodo))
    data.update({'medida': medida, 'fecha': fecha, 'hora': hora, 'periodo': periodo})
    return data

def adaptar_estado(data):
    if not isinstance(data['timestamp'], int):
        data['timestamp'] = int(data['timestamp'])
    return data

def adaptar_autoconsumo(data):
    for key in REQUIRED_AUTOCONSUMO_KEYS:
        if key not in data:
            raise KeyError("Clave requerida {} no encontrada!".format(key))
    data.update({
        'tipoAutoConsumo': str(data['tipoAutoConsumo']),
        'seccion': str(data['seccion']),
        'subseccion': str(data['subseccion'].lower()),
        'potenciaInstaladaGeneracion': float(data['potenciaInstaladaGeneracion'])
    })
    return data



# -*- coding: utf-8 -*-
from os import path

CODIGOS_TARIFA = ['30', '31', '62', '63', '64', '65', '21', '2A', '6A', '2T', '3T', '3V', '6V']
CODIGOS_TENSION = ['E0', 'E1', 'E2', 'E3', 'E4', 'E5', 'E6']
CODIGOS_DH = ['G0', 'E3', 'E2', 'E1']
CODIGOS_AUTOCONSUMO = ['31', '32', '33', '41', '42', '51', '52', '53', '54', '55', '56', '61', '62', '63', '64', '71', '72', '73', '74']

def validar_contrato(data):
    if not isinstance(data, dict):
        raise Exception("El formato de datos debe ser un diccionario")

    validar_provincia(data['provincia'])
    validar_municipio(data['municipio'])
    validar_cups(data['cups'])
    validar_tension(data['tensionConexion'])
    validar_tarifa(data['tarifaAcceso'])
    validar_dh(data['discriminacionHoraria'])
    validar_tipo_punto(data['tipoPunto'])
    validar_control_potencia(data['modoControlPotencia'])
    validar_nif(data['nif'])
    validar_cnae(data['CNAE'])
    if 'potenciasContratadas' in data:
        validar_potencias_contratadas(data['potenciasContratadas'])
    if 'cau' in data:
        validar_cau(data['cau'])
        validar_coeficiente_reparto(data['coeficienteReparto'])

def validar_autoconsumo(data):
    if not isinstance(data, dict):
        raise Exception("El formato de datos debe ser un diccionario")

    validar_cau(data['cau'])
    validar_tipo_autoconsumo(data['tipoAutoConsumo'])
    validar_seccion(data['seccion'])

def validar_provincia(provincia):
    provincias, municipios = read_provincias_municipios()
    if provincia not in provincias:
        raise Exception('El codigo provincia {} no existe en el INE'.format(provincia))

def validar_municipio(municipio):
    provincias, municipios = read_provincias_municipios()
    if municipio not in municipios:
        raise Exception('El codigo municipio {} no existe en el INE'.format(municipio))

def validar_cups(cups):
    if len(cups) != 22:
        raise Exception('El cups debe tener 22 caracteres')

def validar_tension(tension):
    if not isinstance(tension, str):
        raise TypeError("Tension {} incorrecta".format(tension))
    if tension.upper() not in CODIGOS_TENSION:
        raise Exception("Codigo tension {} no catalogada en las tablas de tension".format(tension))

def validar_tarifa(tarifa):
    if not isinstance(tarifa, str):
        raise TypeError("Tarifa {} incorrecta".format(tarifa))
    if tarifa.upper() not in CODIGOS_TARIFA:
        raise Exception("Codigo tarifa {} no catalogada en las tablas de tarifa".format(tarifa))

def validar_dh(dh):
    if not isinstance(dh, str):
        raise TypeError("Discriminacion horaria {} incorrecta".format(dh))
    if dh.upper() not in CODIGOS_DH:
        raise Exception("Codigo discriminacion horaria {} no catalogada en las tablas de discriminacion horaria -> {}".format(dh, CODIGOS_DH))

def validar_tipo_punto(tipo_punto):
    if int(tipo_punto) > 5 or int(tipo_punto) < 1:
        raise Exception("Codigo tipo de punto {} incorrecto -> 1, 2, 3, 4, 5".format(tipo_punto))

def validar_control_potencia(control_potencia):
    if not isinstance(control_potencia, int):
        raise TypeError("Modo control potencia {} incorrecto".format(control_potencia))
    if control_potencia not in (1, 2):
        raise Exception("Modo control potencia {} incorrecto -> 1: Maximetro, 2: ICP")

def validar_nif(nif):
    if len(nif) < 9:
        raise Exception("Longitud NIF/CIF {} incorrecto".format(nif))

def validar_potencias_contratadas(potencias_contratadas):
    if not isinstance(potencias_contratadas, (list, tuple)):
        raise Exception("Las potencias contratadas deben enviarse en formato lista")
    for pot in potencias_contratadas:
        if not isinstance(pot, float):
            raise Exception("Las potencias contratadas deben enviarse en kWh y en decimales")

def validar_cnae(cnae):
    if cnae and len(cnae) != 4:
        raise Exception("Longitud CNAE {} incorrecta".format(cnae))

def validar_cau(cau):
    if len(cau) != 26:
        raise Exception('El CAU debe tener 26 caracteres')

def validar_tipo_autoconsumo(tipo_autoconsumo):
    if not isinstance(tipo_autoconsumo, str):
        raise TypeError("Tipo autoconsumo {} incorrecto, debe ser un string".format(tipo_autoconsumo))
    if tipo_autoconsumo.upper() not in CODIGOS_AUTOCONSUMO:
        raise Exception(
            "Tipo autoconsumo {} no catalogado en las tablas -> {}".format(tipo_autoconsumo, CODIGOS_AUTOCONSUMO))

def validar_seccion(seccion):
    if not int(seccion) in (1, 2):
        raise Exception(
            "Seccion incorrecta {}: Solo se permite seccion 1 Sin excedentes o 2 Con excedentes".format(seccion))

def validar_coeficiente_reparto(coeficiente_reparto):
    if not isinstance(coeficiente_reparto, (int, float)):
        raise TypeError("El formato de Coeficiente de reparto {} es incorrecto.".format(coeficiente_reparto))

def read_provincias_municipios():
    import csv
    municipios = []
    provincias = []
    abs_route_fname = path.join(
        path.dirname(path.realpath(__file__)), 'data/20codmun.csv'
    )
    with open(abs_route_fname) as csvfile:
        for CODAUTO, CPRO, CMUN, DC, NOMBRE in csv.reader(csvfile, delimiter=';'):
            provincias.append(CPRO)
            municipios.append(CMUN)
    return provincias, municipios



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
from datadis import __version__, __author__

with open('requirements.txt', 'r') as f:
    requirements = f.readlines()

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name='datadis-aseme',
    version=__version__,
    description="Herramienta gestión API ASEME DATADIS",
    long_description=long_description,
    long_description_content_type="text/markdown",
    provides=['datadis'],
    packages=find_packages(),
    install_requires=requirements,
    license='BSD 3-Clause License',
    author=__author__,
    author_email='devel@gisce.net',
    url = 'https://github.com/gisce/datadis-aseme',
)
